#include <vozrast.mq4>
#include <astro_swiss.mq4>


//
int get_period_seconds(int initial_shift, int Body, int iflag)
{
   double initial_gradus = get_astro_gradus(initial_shift, Body, iflag);
   datetime initial_time = Time[initial_shift];
   datetime current_time = initial_time;
   
   check_friday(initial_shift);
   
   int shift = initial_shift;
   double past_gradus = initial_gradus;
   double vozrast = 0;
   double add_360 = 0.0;
   double current_gradus;
   
   while(vozrast < 359.99)
   {
     current_time = current_time + Period()*60;
     current_gradus = get_astro_gradus_d(current_time, Body, iflag);
     if(current_gradus < past_gradus) add_360 = add_360 + 1.0;
     vozrast = current_gradus + add_360*360.0 - initial_gradus;
     past_gradus = current_gradus;
   }
   return(current_time-initial_time);
}


//
//  360/edgenumber     
//
int get_astro_period_bars(int Body, int iflag, double edge_number = -1)
{
   double result = 100;
   double body_period = 0;
   
   if(Body == -1)
   {
     body_period = 1*24.0;
     //if(edgenumber > 0) { body_period = body_period*edgenumber; }
   }
   else
   if(Body == -2)
   {
     body_period = 5*24.0;
     //if(edgenumber > 0) { body_period = body_period*edgenumber; }
   }
   else
   {
      body_period = get_astro_period_hours(Body,iflag); 
   }
   
   /*
   if(Period() == PERIOD_D1)
   {
     body_period = body_period*(5.0/7.0);
   }
   */
   
   if(edge_number > 0)
   {
     body_period = body_period/edge_number;
   }
   
   //
   if(body_period > 0)
   {
     result = body_period/(Period()/60.0);
   }
   
   return(MathRound(result));
}

//
//  
//
int get_astro_period_hours(int Body, int iflag)
{
   int body_period = 0;
   
   if((Body == SE_MOON)&&(iflag == 0))
   {
     body_period = 27.0*24.0; 
   }
   else
   if((Body == SE_MERCURY)&&(iflag == 8))
   {
     //body_period = 58.0*24.0;
     body_period = 2111; //in hours
   }
   else
   if((Body == SE_VENUS)&&(iflag == 8))
   {
     body_period = 225.0*24.0;
   }
   else
   if((Body == SE_SUN)&&(iflag == 0))
   {
     body_period = 365.0*24.0;
   }
   else
   if((Body == SE_EARTH)&&(iflag == 8))
   {
     body_period = 365.0*24.0;
   }
   else
   if((Body == SE_MARS)&&(iflag == 8))
   {
     body_period = 687.0*24.0;
   }
   else
   if((Body == SE_JUPITER)&&(iflag == 8))
   {
     body_period = 4333.0*24.0;
   }
   else
   if((Body == SE_SATURN)&&(iflag == 8))
   {
     body_period = 10742.0*24.0;
   }
   
   return(body_period);
}

// date_now, date_calculate < date_now
/*
int get_circlenumber(datetime a, datetime b, int Body, int iflag)
{
   //
   if(b >= a)
   {
     return(-1);
   }
   
   double period360 = get_astro_period_hours(Body, iflag)*60*60;
   
   double delta = MathAbs(a - b);
   int number = MathCeil(delta/period360);
   return(number);
}
*/




//
//     .
//
//  =-1     360
//  =-2     360
//
double get_astro_gradus(int shift, int Body, int iflag)//, int edgenumber = -1;)
{
   datetime d;
   if(shift >= 0)
   {
     d = Time[shift];
   }
   else
   {
     d = Time[0] + MathAbs(shift)*Period()*60;
   }
   
   double current_gradus = 0;
   
   if(Body == -1)
   {
     current_gradus = TimeHour(d)*(360.0/24.0) + TimeMinute(d)*(360.0/24.0)/60.0; // 
   }
   else
   if(Body == -2)
   {
     current_gradus = ((TimeDayOfWeek(d)-1)*24.0 + TimeHour(d) )*(360.0/(5.0*24.0)); //  //(1)--(5)*24
   }
   else
   if(Body == -3)
   {
     current_gradus = astro_swe_houses(d);
   }
   else
   {
     double longtitude, latitude;
     double longtitude2, latitude2;
     double distance, distance2;
     double speed;
     astro_swe_calc(d, Body, iflag, longtitude, latitude, distance, speed);
     current_gradus = longtitude;
   }
   
   return(current_gradus);
}


double get_astro_gradus_d(datetime d, int Body, int iflag)//, int edgenumber = -1;)
{
   double current_gradus = 0;
   
   if(Body == -1)
   {
     current_gradus = TimeHour(d)*(360.0/24.0) + TimeMinute(d)*(360.0/24.0)/60.0; // 
   }
   else
   if(Body == -2)
   {
     current_gradus = ((TimeDayOfWeek(d)-1)*24.0 + TimeHour(d) )*(360.0/(5.0*24.0)); //  //(1)--(5)*24
   }
   else
   if(Body == -3)
   {
     current_gradus = astro_swe_houses(d);
   }
   else
   {
     double longtitude, latitude;
     double longtitude2, latitude2;
     double distance, distance2;
     double speed;
     astro_swe_calc(d, Body, iflag, longtitude, latitude, distance, speed);
     current_gradus = longtitude;
   }
   
   return(current_gradus);
}


//
//  (,   )  initial_gradus
//
int edge1number = 0;
int get_astro_number(int shift, int Body, int iflag, int EdgeNumber, double initial_gradus)
{
   double current_gradus = get_astro_gradus(shift, Body, iflag);
   double angle_value = current_gradus - initial_gradus;
   if(angle_value < 0) { angle_value = angle_value + 360.0; }   
   
   // 1
   if( (EdgeNumber == 1)&&( (iflag ==8)||((Body==1)&&(iflag==0)) ) ) // helio or moon
   {
     int number1 = MathFloor(angle_value/(360.0/2));
     /*
     double past_gradus = get_astro_gradus(shift+1, Body, iflag);
     double past_angle_value = past_angle_value - initial_gradus;
     if(past_angle_value < 0) { past_angle_value = past_angle_value + 360.0; }   
     int number_past = MathFloor(past_angle_value/(360.0/2));
     if((MathMod(number1,2) == 0)&&(number1!=number_past))
     {
       edge1number = edge1number + 1;
     }
     */
     if(MathMod(number1,2) == 0)
     {
       return(edge1number);
     }
     else
     {
       return(edge1number-1);
     }
   }
   
   // normal
   int number = MathFloor(angle_value/(360.0/EdgeNumber));
   
   return(number);
}


